import React from 'react'
import { ChatMessageInputElementProps } from '../models'
import './ChatMessageInput.scss'

export const ChatMessageInput: React.FC<ChatMessageInputElementProps> = React.forwardRef<
  HTMLTextAreaElement,
  ChatMessageInputElementProps
>(({ currentChatMessage, onChatMessageChanged, sendMessage, placeholderText = 'Type a message…' }, ref) => {
  const MESSAGE_MAX_LENGTH = 1000

  const keyDownHandler = (evt: React.KeyboardEvent<HTMLElement>) => {
    if (evt.key === 'Enter' && !evt.shiftKey) {
      evt.preventDefault()
      sendMessage && sendMessage()
    }
  }

  return (
    <textarea
      ref={ref}
      className='chat-input__message-textarea'
      value={currentChatMessage}
      onChange={(evt) => onChatMessageChanged(evt.target.value)}
      onKeyDown={keyDownHandler}
      placeholder={placeholderText}
      maxLength={MESSAGE_MAX_LENGTH}
    />
  )
})
