import React from 'react'
import { ChatMessageAttachmentElementProps } from '../models'
import { useChatAttachment } from '../hooks'
import './ChatMessageAttachment.scss'

export const ChatMessageAttachment: React.FC<ChatMessageAttachmentElementProps> = ({
  attachment,
  getAttachmentLocation,
}) => {
  const { anchorRef, isAttachmentLoading, hasAttachmentError, attachmentUrl, downloadAttachment } = useChatAttachment(
    attachment,
    getAttachmentLocation
  )

  return (
    <div className='chat-message-attachment__container'>
      <div className='chat-message-attachment__name'>{`📎 ${attachment.fileName}`}</div>
      <button
        className='chat-message-attachment__download-button'
        disabled={isAttachmentLoading}
        onClick={downloadAttachment}
      >
        Download
      </button>
      <a
        ref={anchorRef}
        className='chat-message-attachment__hidden-link'
        href={attachmentUrl}
        download={attachment.fileName}
        target='_blank'
        rel='noreferrer noopener'
      >
        {' '}
      </a>
      {hasAttachmentError && <div>Error while downloading the attachment.</div>}
    </div>
  )
}
