import React from 'react'
import { ParticipantsGridView } from '../ParticipantsGridView'
import { ParticipantsSpeakerView } from '../ParticipantsSpeakerView'
import { useZocksStore } from '../../../app'
import { ParticipantsViewType, selectCurrentParticipantsView, selectIsScreenSharing } from '../../../store'
import { ParticipantsScreenShareView } from '../ParticipantsScreenShareView'
import { ParticipantAudio } from '../ParticipantAudio'
import { ParticipantsProps } from '../model'
import { Participant } from '../../Participant'
import { ScreenShare } from '../../ScreenShare'
import './Participants.scss'

export const Participants: React.FC<ParticipantsProps> = ({
  gridPageSize = 25,
  speakerPageSize = 4,
  screenSharePageSize = 4,
  participantElement,
  screenShareElement,
}) => {
  const isScreenSharing = useZocksStore(selectIsScreenSharing)
  const currentView = useZocksStore(selectCurrentParticipantsView)

  const ParticipantElement = participantElement ?? Participant
  const ScreenShareElement = screenShareElement ?? ScreenShare

  const getCurrentView = () => {
    switch (true) {
      case isScreenSharing:
        return (
          <ParticipantsScreenShareView
            pageSize={screenSharePageSize}
            participantElement={ParticipantElement}
            screenShareElement={ScreenShareElement}
          />
        )
      case currentView === ParticipantsViewType.Grid:
        return <ParticipantsGridView pageSize={gridPageSize} participantElement={ParticipantElement} />
      case currentView === ParticipantsViewType.Speaker:
        return <ParticipantsSpeakerView pageSize={speakerPageSize} participantElement={ParticipantElement} />
    }
  }

  return (
    <div className='participants'>
      {getCurrentView()}
      <ParticipantAudio />
    </div>
  )
}
