import React from 'react'
import { useZocksStore } from '../../../app'
import { selectLocalParticipant, selectScreenSharingParticipant } from '../../../store'
import { Participant } from '../../Participant'
import { ScreenShare } from '../../ScreenShare'
import { usePagedParticipants, useSortRemoteParticipantsBySpeakers } from '../hooks'
import { ScreenShareViewProps } from '../model'
import './ParticipantsScreenShareView.scss'

export const ParticipantsScreenShareView: React.FC<ScreenShareViewProps> = ({
  pageSize = 4,
  participantElement,
  screenShareElement,
}) => {
  const localParticipant = useZocksStore(selectLocalParticipant)
  const screenSharingParticipant = useZocksStore(selectScreenSharingParticipant)

  const { remoteParticipantsSortedBySpeakers } = useSortRemoteParticipantsBySpeakers()

  const allParticipants = [localParticipant, ...remoteParticipantsSortedBySpeakers]
  const { displayedParticipants, hasNextPage, hasPreviousPage, nextPage, previousPage } = usePagedParticipants(
    allParticipants,
    pageSize
  )

  const ParticipantElement = participantElement ?? Participant
  const ScreenShareElement = screenShareElement ?? ScreenShare

  return (
    <div className='participants-screen-share-view'>
      <div className='participants-screen-share-view__current-share'>
        {screenSharingParticipant && <ScreenShareElement participant={screenSharingParticipant} />}
      </div>
      {hasPreviousPage && <PreviousButton onClick={previousPage} />}
      <div className='participants-screen-share-view__participants-container'>
        <div className='participants-screen-share-view__participants'>
          {displayedParticipants.map((participant) => (
            <ParticipantElement key={participant.id} participant={participant} />
          ))}
        </div>
      </div>
      {hasNextPage && <NextButton onClick={nextPage} />}
    </div>
  )
}

const PreviousButton: React.FC<{ onClick: () => void }> = ({ onClick }) => (
  <button
    className='participants-screen-share-view__button participants-screen-share-view__previous-button'
    onClick={onClick}
    aria-label='Previous'
  >
    <div className='participants-screen-share-view__previous-button-icon' />
  </button>
)

const NextButton: React.FC<{ onClick: () => void }> = ({ onClick }) => (
  <button
    className='participants-screen-share-view__button participants-screen-share-view__next-button'
    onClick={onClick}
    aria-label='Next'
  >
    <div className='participants-screen-share-view__next-button-icon' />
  </button>
)
