import React from 'react'
import classNames from 'classnames'
import { useZocksStore } from '../../../app'
import { selectActiveSpeakers, selectLocalParticipant, selectRemoteParticipants } from '../../../store'
import { ParticipantsList } from '../ParticipantsList'
import { OnlineParticipantsListProps, ParticipantsListElementProps } from '../model'
import '../ParticipantsList.scss'

export const OnlineParticipantsList: React.FC<OnlineParticipantsListProps> = ({ dismiss, participantElement }) => {
  const localParticipant = useZocksStore(selectLocalParticipant)
  const remoteParticipants = useZocksStore(selectRemoteParticipants)
  const allParticipants = [localParticipant, ...remoteParticipants]

  const ParticipantElement = participantElement ?? OnlineParticipantsListElement

  return <ParticipantsList participants={allParticipants} dismiss={dismiss} participantElement={ParticipantElement} />
}

const OnlineParticipantsListElement: React.FC<ParticipantsListElementProps> = ({ participant }) => {
  const activeSpeakers = useZocksStore(selectActiveSpeakers)
  const isSpeaking = activeSpeakers.some((participantId) => participantId === participant.id)
  const noAudio = participant.tracks.audioTracks.every((t) => t.isMuted)
  const noVideo = participant.tracks.videoTracks.every((t) => t.isMuted)

  return (
    <div className='participants-list__participant-container'>
      <div className='participants-list__participant-avatar'>{participant.name?.slice(0, 1) || ''}</div>
      <div className='participants-list__participant-name'>{participant.name}</div>
      <div
        className={classNames({
          'participants-list__participant-video-icon': !noVideo,
          'participants-list__participant-no-video-icon': noVideo,
        })}
      />
      <div
        className={classNames({
          'participants-list__participant-audio-icon': !noAudio,
          'participants-list__participant-no-audio-icon': noAudio,
          'participants-list__participant-speaking': isSpeaking,
        })}
      />
    </div>
  )
}
