import React, { useMemo } from 'react'
import { VirtualBackgroundType } from '@zocks-communications/client-sdk-js'
import { MenuItem, SelectButton } from '../common'
import { VirtualBackgroundSelectorProps } from './models'
import './VirtualBackgroundSelector.scss'

export const VirtualBackgroundSelector: React.FC<VirtualBackgroundSelectorProps> = ({
  availableVirtualBackgrounds,
  selectVirtualBackground,
  virtualBackgroundIsLoading,
}) => {
  const NO_BACKGROUND = 'None'

  const backgroundMenuItems = useMemo(() => {
    return [{ label: NO_BACKGROUND }, ...availableVirtualBackgrounds.map((background) => ({ label: background }))]
  }, [availableVirtualBackgrounds])

  const selectMenuItem = (menuItem: MenuItem) => {
    const { label } = menuItem
    if (label === NO_BACKGROUND) {
      selectVirtualBackground(null)
    } else {
      selectVirtualBackground(label as VirtualBackgroundType)
    }
  }

  return (
    <div className='virtual-background-select-container'>
      <div className='virtual-background-select-text'>Background</div>
      <SelectButton
        className='virtual-background-select-button'
        menuItems={backgroundMenuItems}
        onMenuItemSelected={selectMenuItem}
        disabled={virtualBackgroundIsLoading}
        buttonLabel='Background'
      />
    </div>
  )
}
