import React, { useState } from 'react'
import classNames from 'classnames'
import { Popover } from 'react-tiny-popover'
import { MenuItem, SelectMenuElementProps } from './models'
import './SelectButton.scss'

export const SelectButton: React.FC<{
  menuItems: MenuItem[]
  onMenuItemSelected?: (item: MenuItem) => void
  disabled?: boolean
  className?: string
  buttonLabel?: string
  selectMenuElement?: React.FC<SelectMenuElementProps>
}> = ({ menuItems, onMenuItemSelected, disabled, className, buttonLabel, selectMenuElement }) => {
  const [popoverVisible, setPopoverVisible] = useState(false)

  const hideMenu = () => {
    setPopoverVisible(false)
  }

  const handleMenuItemClick = (item: MenuItem) => {
    hideMenu()
    onMenuItemSelected && onMenuItemSelected(item)
  }

  const SelectMenuElement = selectMenuElement ?? SelectMenu

  return (
    <div className={className}>
      <Popover
        isOpen={popoverVisible}
        positions={['top']}
        content={<SelectMenuElement menuItems={menuItems} onMenuItemClick={handleMenuItemClick} hideMenu={hideMenu} />}
        onClickOutside={() => setPopoverVisible(false)}
      >
        <div
          className={classNames('select-button__button-container', {
            disabled: disabled,
          })}
        >
          <button
            disabled={disabled}
            className='select-button__button'
            onClick={() => setPopoverVisible(!popoverVisible)}
            aria-label={buttonLabel}
          >
            <div className='select-button__button-icon' />
          </button>
        </div>
      </Popover>
    </div>
  )
}

const SelectMenu: React.FC<SelectMenuElementProps> = ({ menuItems, onMenuItemClick }) => {
  return (
    <div className='select-button__popover-container'>
      <ul className='select-button__popover-list'>
        {menuItems?.map((item, i) => {
          return (
            <li className='select-button__popover-item' key={i} onClick={() => onMenuItemClick(item)}>
              {item.label}
            </li>
          )
        })}
      </ul>
    </div>
  )
}
