import React from 'react'
import { VirtualBackgroundType } from '@zocks-communications/client-sdk-js'
import { SelectMenuElementProps } from '../../../components'

export const createVideoControlMenu =
  (
    currentVirtualBackground?: VirtualBackgroundType | null,
    selectVirtualBackground?: (virtualBackgroundType: VirtualBackgroundType | null) => void
  ): React.FC<SelectMenuElementProps> =>
  ({ menuItems, onMenuItemClick, hideMenu }) => {
    const toggleBlur = () => {
      hideMenu()
      if (currentVirtualBackground) {
        selectVirtualBackground && selectVirtualBackground(null)
      } else {
        selectVirtualBackground && selectVirtualBackground(VirtualBackgroundType.BLUR)
      }
    }

    return (
      <div className='select-button__popover-container'>
        <ul className='select-button__popover-list'>
          <li className='select-button__popover-item camera-settings-menu-item'>Camera Settings</li>
          {menuItems.length > 0
            ? menuItems.map((item, i) => {
                return (
                  <li className='select-button__popover-item' key={i} onClick={() => onMenuItemClick(item)}>
                    {item.label}
                    {item.selected ? <i className='icon-check-mark' /> : null}
                  </li>
                )
              })
            : null}
          <li className='select-button__popover-item blur-menu-item' onClick={toggleBlur}>
            {currentVirtualBackground ? (
              <>
                <i className='icon-virtual-background' />
                Unblur My Background
              </>
            ) : (
              <>
                <i className='icon-background-blur' />
                Blur My Background
              </>
            )}
          </li>
        </ul>
      </div>
    )
  }
